def defaults args
  args.state.enemies ||= []
  args.state.bullets ||= []
end

def render args
  args.outputs.borders << args.state.enemies.map(&:rect)
  args.outputs.labels  << args.state.enemies.flat_map do |enemy|
    [
      [enemy.x + 4, enemy.y + 29, "id: #{enemy.entity_id}", -3, 0],
      [enemy.x + 4, enemy.y + 17, "created_at: #{enemy.created_at}", -3, 0]
    ]
  end

  args.outputs.solids << args.state.bullets.map(&:rect)
end

def tick args
  defaults args
  render args
  calc args
  process_inputs args
end

def add_new_enemies_if_needed args
  return if args.state.enemies.length >= 10
  return unless args.state.bullets.length == 0

  args.state.enemies += (10 - args.state.enemies.length).map do
    args.state.new_entity(:enemy) do |e|
      e.x = 640 + 500 * rand
      e.y = 600 * rand + 50
      e.rect = [e.x, e.y, 130, 30]
    end
  end
end

def move_bullets args
  args.state.bullets.each do |bullet|
    bullet.x += bullet.speed
    bullet.y += bullet.speed.*(0.25).randomize(:ratio, :sign)
    bullet.rect = [bullet.x, bullet.y, bullet.size, bullet.size]
  end
end

def calculate_collisions args
  args.state.bullets.each do |bullet|
    args.state.enemies.each do |enemy|
      if !bullet.exploded && bullet.rect.intersects_rect?(enemy.rect)
        bullet.exploded = true
        enemy.dead = true
      end
    end
  end

  args.state.bullets = args.state.bullets.reject(&:exploded)
  args.state.enemies = args.state.enemies.reject(&:dead)
end

def remove_bullets_of_screen args
  args.state.bullets = args.state.bullets.reject { |bullet| bullet.x > 1280 }
end

def calc args
  add_new_enemies_if_needed args
  move_bullets args
  calculate_collisions args
  remove_bullets_of_screen args
end

def fire_bullet args
  return unless args.inputs.mouse.click
  args.state.bullets << args.state.new_entity(:bullet) do |bullet|
    bullet.y = args.inputs.mouse.click.point.y
    bullet.x = 0
    bullet.size = 10
    bullet.speed = 10 * rand + 2
    bullet.rect = [bullet.x, bullet.y, bullet.size, bullet.size]
  end
end

def process_inputs args
  fire_bullet args
end
