class TicTacToe
  attr_accessor :_, :state, :outputs, :inputs, :grid, :gtk

  def filled_spaces
    state.space_combinations
      .reject { |x, y| !state.spaces[x][y].piece }
      .map do |x, y|
        if block_given?
          yield x, y, state.spaces[x][y]
        else
          [x, y, state.spaces[x][y]]
        end
    end
  end

  def all_spaces
    if !block_given?
      state.space_combinations.map do |x, y|
        [x, y, state.spaces[x][y]]
      end
    else
      state.space_combinations.map do |x, y|
        yield x, y, state.spaces[x][y]
      end
    end
  end

  def label x, y, value
    [x, y + 10, value, 20, 1, 0, 0, 0]
  end

  def render_board
    square_size = 80
    board_left = grid.w_half - square_size * 1.5
    board_top  = grid.h_half - square_size * 1.5
    outputs.borders << all_spaces do |x, y, space|
      space.border ||= [
        board_left + x.add(1) * square_size,
        board_top  + y.add(1) * square_size,
        square_size,
        square_size
      ]
    end

    outputs.labels << filled_spaces do |x, y, space|
      label board_left + x.add(1) * square_size + square_size.fdiv(2),
            board_top  + y.add(1) * square_size + square_size - 20,
            space.piece
    end

    outputs.labels << if state.x_won
                        label grid.w_half, grid.top - 80, "x won"
                      elsif state.o_won
                        label grid.w_half, grid.top - 80, "o won"
                      elsif state.draw
                        label grid.w_half, grid.top - 80, "a draw"
                      else
                        label grid.w_half, grid.top - 80, "turn: #{state.current_turn}"
                      end
  end

  def won? piece
    won = [[-1, 0, 1]].product([-1, 0, 1]).map do |xs, y|
      horizontal_match = state.spaces[xs[0]][y].piece == piece &&
                         state.spaces[xs[1]][y].piece == piece &&
                         state.spaces[xs[2]][y].piece == piece

      vertical_match = state.spaces[y][xs[0]].piece == piece &&
                       state.spaces[y][xs[1]].piece == piece &&
                       state.spaces[y][xs[2]].piece == piece

      horizontal_match || vertical_match
    end

    won << (state.spaces[-1][-1].piece == piece &&
            state.spaces[ 0][ 0].piece == piece &&
            state.spaces[ 1][ 1].piece == piece)

    won << (state.spaces[ 1][-1].piece == piece &&
            state.spaces[ 0][ 0].piece == piece &&
            state.spaces[-1][ 1].piece == piece)

    won.reject_false.any?
  end

  def determine_winner
    state.x_won = won? :x
    state.o_won = won? :o
    state.draw = true if filled_spaces.length == 9 && !state.x_won && !state.o_won
    state.game_over = state.x_won || state.o_won || state.draw
  end

  def input_place_piece
    return if state.game_over

    __, __, space = all_spaces.find do |__, __, space|
      inputs.mouse.click.point.inside_rect?(space.border) && !space.piece
    end

    return unless space
    space.piece = state.current_turn
    state.current_turn = state.current_turn == :x ? :o : :x
  end

  def input_restart_game
    return unless state.game_over
    gtk.reset
  end

  def input_board
    return unless inputs.mouse.click
    input_place_piece
    input_restart_game
    determine_winner
  end

  def tick
    state.current_turn ||= :x
    state.space_combinations = [-1, 0, 1].product([-1, 0, 1]).to_a
    render_board
    input_board
  end
end

$tic_tac_toe = TicTacToe.new

def tick args
  $tic_tac_toe._       = args
  $tic_tac_toe.state   = args.state
  $tic_tac_toe.outputs = args.outputs
  $tic_tac_toe.inputs  = args.inputs
  $tic_tac_toe.grid    = args.grid
  $tic_tac_toe.gtk     = args.gtk
  $tic_tac_toe.tick
end
