# Along with outputs, inputs are also an essential part of video game development
# DragonRuby can take input from keyboards, mouse, and controllers.
# This sample app will cover keyboard input.

# args.inputs.keyboard.key_up.a will check to see if the a key has been pressed
# This will work with the other keys as well


def tick args
  # Notice how small_font accounts for all the remaining parameters
  args.outputs.labels << [460, row_to_px(args, 0), "Current game time: #{args.game.tick_count}", small_font]
  args.outputs.labels << [460, row_to_px(args, 2), "Keyboard input: args.inputs.keyboard.key_up.h", small_font]
  args.outputs.labels << [460, row_to_px(args, 3), "Press \"h\" on the keyboard.", small_font]

  # Input on a specifc key can be found through args.inputs.keyboard.key_up followed by the key
  if args.inputs.keyboard.key_up.h
    args.game.h_pressed_at = args.game.tick_count
  end

  # This code simplifies to if args.game.h_pressed_at has not been initialized, set it to false
  args.game.h_pressed_at ||= false

  if args.game.h_pressed_at
    args.outputs.labels << [460, row_to_px(args, 4), "\"h\" was pressed at time: #{args.game.h_pressed_at}", small_font]
  else
    args.outputs.labels << [460, row_to_px(args, 4), "\"h\" has never been pressed.", small_font]
  end
end

def small_font
  # This method provides some values for the construction of labels
  # Specifically, Size, Alignment, & RGBA
  # This makes it so that custom parameters don't have to be repeatedly typed.
  # Additionally "small_font" provides programmers with more information than some numbers
  [-2, 0, 0, 0, 0, 255]
end

def row_to_px args, row_number
  # This takes a row_number and converts it to pixels DragonRuby understands.
  # Row 0 starts 5 units below the top of the grid
  # Each row afterward is 20 units lower
  args.grid.top.shift_down(5).shift_down(20 * row_number)
end
