def defaults _
  _.state.notes ||= []
  _.state.click_feedbacks ||= []
  _.state.current_level ||= 1
  _.state.times_wrong ||= 0
end

def piano_button _, note, position
  _.state.new_entity(:button) do |b|
    b.label  =  [460 + 40.mult(position), _.grid.h * 0.4, "#{note}", 0, 1, 0, 0, 0]
    b.border =  _.state.with_meta([460 + 40.mult(position) - 20, _.grid.h * 0.4 - 32, 40, 40], note)
  end
end

def render _
  _.outputs.labels << [_.grid.w_half, _.grid.top.shift_down(40), "Hole #{_.state.current_level} of 9", 0, 1, 0, 0, 0]

  render_score _

  if _.state.game_over
    _.state.play_again_border ||= _.state.with_meta([560, _.grid.h * 3 / 4 - 40, 160, 60], 'again')
    _.outputs.labels <<  [_.grid.w_half, _.grid.h * 3 / 4, "Play Again", 0, 1, 0, 0, 0]
    _.outputs.borders << _.state.play_again_border
  else
    _.state.play_note_border ||= _.state.with_meta([560, _.grid.h * 3 / 4 - 40, 160, 60], 'play')
    _.outputs.labels <<  [_.grid.w_half, _.grid.h * 3 / 4, "Play Note ##{_.state.current_level}", 0, 1, 0, 0, 0]
    _.outputs.borders << _.state.play_note_border
  end

  return if _.state.game_over

  _.outputs.labels <<   [_.grid.w_half, 400, "I think the note is a(n)...",  0, 1, 0, 0, 0]

  available_notes.each_with_index do |note, i|
    _.state.notes[i] ||= piano_button(_, note, i + 1)
    _.outputs.labels <<   _.state.notes[i].label
    _.outputs.borders <<  _.state.notes[i].border
  end

  _.outputs.solids << _.state.click_feedbacks.map { |c| c.solid }
end

def available_notes
  [:C3, :D3, :E3, :F3, :G3, :A3, :B3, :C4]
end

def calc _
  _.state.target_note ||= available_notes.sample
  _.state.click_feedbacks.each    { |c| c.solid[-1] -= 5 }
  _.state.click_feedbacks.reject! { |c| c.solid[-1] <= 0 }
end

def render_score _
  if _.state.times_wrong == 0
    _.outputs.labels << [_.grid.w_half, _.grid.top.shift_down(80), "Score: PAR", 0, 1, 0, 0, 0]
  else
    _.outputs.labels << [_.grid.w_half, _.grid.top.shift_down(80), "Score: +#{_.state.times_wrong}", 0, 1, 0, 0, 0]
  end
end

def queue_click_feedback _, x, y, w, h, *color
  _.state.click_feedbacks << _.state.new_entity(:click_feedback) do |c|
    c.solid =  [x, y, w, h, *color, 255]
  end
end

def input_mouse _
  return unless _.inputs.mouse.click

  button_clicked = _.outputs.borders.find_all do |b|
    _.inputs.mouse.click.point.inside_rect? b
  end.reject {|b| !_.state.meta(b)}.first

  return unless button_clicked

  queue_click_feedback _,
                       button_clicked.x,
                       button_clicked.y,
                       button_clicked.w,
                       button_clicked.h,
                       150, 100, 200

  if _.state.meta(button_clicked) == 'play'
    _.outputs.sounds << "sounds/#{_.state.target_note}.wav"
  elsif _.state.meta(button_clicked) == 'again'
      _.state.target_note = nil
      _.state.current_level = 1
      _.state.times_wrong = 0
      _.state.game_over = false
  else
    _.outputs.sounds << "sounds/#{_.state.meta(button_clicked)}.wav"
    if _.state.meta(button_clicked).to_sym == _.state.target_note
      _.state.target_note = nil

      if _.state.current_level < 9
        _.state.current_level += 1
      else
        _.state.game_over = true
      end

      queue_click_feedback _, 0, 0, _.grid.w, _.grid.h, 100, 200, 100
    else
      _.state.times_wrong += 1
      queue_click_feedback _, 0, 0, _.grid.w, _.grid.h, 200, 100, 100
    end
  end
end

def tick args
  defaults args
  render args
  calc args
  input_mouse args
end
