DURATION       = 1.seconds
DEBOUNCE       = 1.seconds
GRID_SIZE      = 600
DOT_RADIUS     = 20
DOT_DIAMETER   = DOT_RADIUS * 2
MIDPOINT       = GRID_SIZE.half
OFFSET_X       = (1280 - GRID_SIZE).half
OFFSET_Y       = (720 - GRID_SIZE).half

def tick args
  args.outputs.borders << [grid_rect(0, 0, GRID_SIZE, GRID_SIZE), 200, 200, 200]
  args.outputs.lines   << [gridline(0, MIDPOINT, GRID_SIZE, MIDPOINT), 200, 200, 200]
  args.outputs.lines   << [gridline(MIDPOINT, 0, MIDPOINT, GRID_SIZE), 200, 200, 200]
  reset_graph args
  easing_samples = [
    ['identity',        [:identity]],
    ['smooth start 2',  [:quad    ]],
    ['smooth start 3',  [:cube    ]],
    ['smooth start 4',  [:quart   ]],
    ['smooth start 5',  [:quint   ]],

    ['flip',            [:flip               ]],
    ['smooth stop 2',   [:flip,  :quad, :flip]],
    ['smooth stop 3',   [:flip,  :cube, :flip]],
    ['smooth stop 4',   [:flip, :quart, :flip]],
    ['smooth stop 5',   [:flip, :quint, :flip]],

    ['custom', [lambda do |x, start, duration|
                  if x < 0.5
                    start.ease(duration)
                  else
                    start.ease(duration, :flip)
                  end
                end]],
    ['custom 2', [lambda do |x, start, duration|
                    Easing.flip(x) * Easing.identity(x) * 4
                  end]],
  ]

  args.state.current_sample ||= 0

  if args.state.current_sample >= easing_samples.length
    args.state.current_sample = 0
  end

  description, ease_definition = easing_samples[args.state.current_sample]
  graph args, description, ease_definition
end

def animation_done? args
  args.state.start_at.elapsed_time > DURATION + DEBOUNCE
end

def reset_graph args
  args.state.start_at ||= args.state.tick_count
  if animation_done? args
    args.state.start_at = args.state.tick_count + DEBOUNCE
    args.state.current_sample += 1
    args.outputs.static_solids.clear
  end
end

def graph args, description, *ease_definition
  render_y_plot args,
                args.state.start_at.ease(DURATION),
                args.state.start_at.ease(DURATION, *ease_definition),
                description
end

def render_y_plot args, x_ratio, y_ratio, message
  x, y = GRID_SIZE * x_ratio, GRID_SIZE * y_ratio
  args.outputs.labels  << [grid_point(MIDPOINT, GRID_SIZE + DOT_DIAMETER + 10), "#{ message } (#{"%.4f)" % y_ratio}", 0, 1]
  args.outputs.solids  << [grid_rect(x - DOT_RADIUS, y - DOT_RADIUS, DOT_DIAMETER, DOT_DIAMETER),
                           0, 0, 0, 255 * y_ratio]
  args.outputs.borders << [grid_rect(x - DOT_RADIUS, y - DOT_RADIUS, DOT_DIAMETER, DOT_DIAMETER)]
  return unless args.state.tick_count.mod_zero?(0.10.seconds)
  args.outputs.static_solids  << [
    grid_rect(x - DOT_RADIUS.fdiv(4), y - DOT_RADIUS.fdiv(4), DOT_DIAMETER.fdiv(4), DOT_DIAMETER.fdiv(4)),
    0, 0, 0, 100
  ]
end

def grid_point x, y
  [OFFSET_X + x, OFFSET_Y + y]
end

def grid_rect x, y, w, h
  [OFFSET_X + x, OFFSET_Y + y, w, h]
end

def gridline x, y, x2, y2
  [
    OFFSET_X + x, OFFSET_Y + y,
    OFFSET_X + x2, OFFSET_Y + y2
  ]
end
