class StarClass
  attr_accessor :sprite, :x, :y, :r, :g, :b, :alpha, :speed_x, :speed_y

  def initialize
    @x = -1280 * rand
    @y = -720 * rand
    @speed_x = 2 * rand + 1
    @speed_y = 2 * rand + 1
    @r = 255
    @g = 255 * rand
    @b = 255 * rand
    @alpha = 128
    @sprite = [@x, @y, 15, 15, 'sprites/star.png', 0, @alpha, @r, @g, @b]
  end
end

def move_stars args
  args.state.stars.map! do |star|
    star.x = -200 * rand if star.x > 1500
    star.y = -200 * rand if star.y > 800
    star.x += star.speed_x
    star.y += star.speed_y
    star.sprite.x = star.x
    star.sprite.y = star.y
    star
  end
end

def render_stars args
  args.outputs.solids << [0, 0, 1280, 720]

  args.outputs.sprites << args.state.stars.map do |star|
    star.sprite
  end

  args.outputs.labels << [10, 30, "Count: #{args.state.star_count}, Type: #{args.state.option}, FPS: #{args.gtk.current_framerate.to_s.to_i}", 255, 255, 255, 80]
end

def defaults args
  args.outputs.static_background_color = [0, 0, 0]
  args.state.star_count ||= 10
  args.state.option ||= :class
  args.state.stars ||= args.state.star_count.map do
    r = nil
    if args.state.option == :entity
      r = args.state.new_entity(:star) do |star|
        star.x = -1280 * rand
        star.y = -720 * rand
        star.speed_x = 2 * rand + 1
        star.speed_y = 2 * rand + 1
        star.r = 255
        star.g = 255
        star.b = 255
        star.alpha = 128
        star.sprite = [star.x, star.y, 15, 15, 'sprites/star.png', 0, star.alpha, 255, 255 * rand, 255 * rand]
      end
    elsif args.state.option == :strict
      r = args.state.new_entity_strict(:star) do |star|
        star.x = -1280 * rand
        star.y = -720 * rand
        star.speed_x = 2 * rand + 1
        star.speed_y = 2 * rand + 1
        star.r = 255
        star.g = 255
        star.b = 255
        star.alpha = 128
        star.sprite = [star.x, star.y, 15, 15, 'sprites/star.png', 0, star.alpha, 255, 255 * rand, 255 * rand]
      end
    elsif args.state.option == :class
      r = StarClass.new
    end
    r
  end
  args.state.stars ||= []
end

def reset_with count, option
  $gtk.reset
  $gtk.args.state.option = option
  $gtk.args.state.star_count = count
end

def process_inputs args
  mouse_border = [args.inputs.mouse.x - 50, args.inputs.mouse.y - 50, 100, 100, 255, 0, 0]
  args.outputs.borders << mouse_border
  $gtk.reset if args.inputs.keyboard.key_down.r
end

def tick args
  args.gtk.suppress_print_help_when_mouse_clicked = true
  args.gtk.suppress_framerate_warning = true

  if Kernel.global_tick_count == 0
    args.gtk.console.show
    puts "
================================================
                    HELLO!!!!
================================================

This sample app shows the performance differences between using
a very flexible data structure (:entity),
a less flexible but faster data structure (:strict),
and the least flexible but fastest data structure (:class).

To see the differences use the `reset_with SPRITE_COUNT, CATEGORY` method.

For example, the following invocations generate 100 sprites of each data structure type:

    reset_with 100, :entity
    reset_with 100, :strict
    reset_with 100, :class

and these commands generate 1k sprites of each data structure type:

    reset_with 1000, :entity
    reset_with 1000, :strict
    reset_with 1000, :class

================================================
"
    args.gtk.console.current_input_str = 'reset_with 1000, :entity'
  end

  defaults args
  render_stars args
  move_stars args
  process_inputs args
end
