class ProtectThePuppiesFromTheZombies
  attr_accessor :grid, :inputs, :state, :outputs

  def tick
    defaults
    render
    calc
    input
  end

  def defaults
    state.flash_at               ||= 0
    state.zombie_min_spawn_rate  ||= 60
    state.zombie_spawn_countdown ||= random_spawn_countdown state.zombie_min_spawn_rate
    state.zombies                ||= []
    state.killed_zombies         ||= []

    state.player ||= state.new_entity(:player, { x: 640,
                                               y: 360,
                                               attack_angle: 0,
                                               dx: 0,
                                               dy: 0 })
  end

  def random_spawn_countdown minimum
    10.randomize(:ratio, :sign).to_i + 60
  end

  def animation_index at
    at.idiv(3).mod(6)
  end

  def animation_sprite zombie, at = nil
    at ||= zombie.created_at_elapsed
    index = animation_index at
    "sprites/zombie-#{index}.png"
  end

  def render_zombies
    outputs.sprites << state.zombies.map do |z|
      z.sprite = [z.x, z.y, 4 * 3, 8 * 3, animation_sprite(z)].sprite
      z.sprite
    end
  end

  def render_killed_zombies
    outputs.sprites << state.killed_zombies.map do |z|
      z.sprite = [z.x,
                  z.y,
                  4 * 3,
                  8 * 3,
                  animation_sprite(z, z.death_at),
                  0,
                  255 * z.death_at.ease(30, :flip)].sprite

      [z.sprite, [z.sprite.rect, 'sprites/slash.png', 45 + state.player.attack_angle_on_click, z.sprite.a].scale_rect(3, 0.5, 0.5)]
    end
  end

  def render_player
    state.player_sprite = [state.player.x,
                           state.player.y,
                          4 * 3,
                          8 * 3, "sprites/player-#{animation_index(state.player.created_at_elapsed)}.png"]
    outputs.sprites << state.player_sprite
    outputs.solids <<  [state.player.x + state.player.attack_angle.vector_x(60),
                        state.player.y + state.player.attack_angle.vector_y(60),
                        3, 3, 255, 0, 0]
  end

  def render
    outputs.solids << [grid.rect, 100, 100, 100]
    render_zombies
    render_killed_zombies
    render_player
    render_flash
  end

  def render_flash
    return if state.flash_at.elapsed_time > 10
    outputs.primitives << [grid.rect, 255, 255, 255, 255 * state.flash_at.ease(10, :flip)].solid
  end

  def calc_spawn_zombie
    if state.zombie_spawn_countdown > 0
      state.zombie_spawn_countdown -= 1
      return
    end

    state.zombies << state.new_entity(:zombie) do |z|
      if rand > 0.5
        z.x = grid.rect.w.randomize(:ratio)
        z.y = [-10, 730].sample
      else
        z.x = [-10, 1290].sample
        z.y = grid.rect.w.randomize(:ratio)
      end
    end

    state.zombie_spawn_countdown = random_spawn_countdown state.zombie_min_spawn_rate
    state.zombie_min_spawn_rate -= 1
    state.zombie_min_spawn_rate  = state.zombie_min_spawn_rate.greater(0)
  end

  def calc_move_zombies
    state.zombies.each do |z|
      z.y = z.y.towards(360, 0.1)
      z.x = z.x.towards(640, 0.1)
    end
    state.zombies = state.zombies.reject { |z| z.y == 360 && z.x == 640 }
  end

  def calc_kill_zombie
    killed_this_frame = state.zombies.find_all { |z| z.sprite.intersects_rect? state.player_sprite }
    state.zombies = state.zombies - killed_this_frame
    state.killed_zombies += killed_this_frame

    if killed_this_frame.length > 0
      state.flash_at = state.tick_count
    end

    killed_this_frame.each do |z|
      z.death_at = state.tick_count
    end

    state.killed_zombies = state.killed_zombies.reject { |z| state.tick_count - z.death_at > 30 }
  end

  def calc
    calc_spawn_zombie
    calc_move_zombies
    calc_player
    calc_kill_zombie
  end

  def calc_player
    state.player.x += state.player.dx
    state.player.y += state.player.dy

    state.player.dx *= 0.9
    state.player.dy *= 0.9

    state.player.x = state.player.x.lesser(1280).greater(0)
    state.player.y = state.player.y.lesser(720).greater(0)
  end

  def input
    if inputs.keyboard.key_held.a || inputs.keyboard.key_held.left
      state.player.x -= 5
    elsif inputs.keyboard.key_held.d || inputs.keyboard.key_held.right
      state.player.x += 5
    end

    if inputs.keyboard.key_held.w || inputs.keyboard.key_held.up
      state.player.y += 5
    elsif inputs.keyboard.key_held.s || inputs.keyboard.key_held.down
      state.player.y -= 5
    end

    if inputs.mouse.moved
      state.player.attack_angle = inputs.mouse.position.angle_given_point [state.player.x, state.player.y]
    end

    if inputs.mouse.click && state.player.dx < 0.5 && state.player.dy < 0.5
      state.player.attack_angle_on_click = inputs.mouse.position.angle_given_point [state.player.x, state.player.y]
      state.player.attack_angle = state.player.attack_angle_on_click
      state.player.dx = state.player.attack_angle.vector_x(25)
      state.player.dy = state.player.attack_angle.vector_y(25)
    end
  end
end

$protect_the_puppies_from_the_zombies = ProtectThePuppiesFromTheZombies.new

def tick args
  $protect_the_puppies_from_the_zombies.grid    = args.grid
  $protect_the_puppies_from_the_zombies.inputs  = args.inputs
  $protect_the_puppies_from_the_zombies.state    = args.state
  $protect_the_puppies_from_the_zombies.outputs = args.outputs
  $protect_the_puppies_from_the_zombies.tick
end
