def default args
  args.state.sprite.width    = 19
  args.state.sprite.height   = 10
  args.state.sprite.scale    = 4
  args.state.max_speed       = 5
  args.state.x             ||= 100
  args.state.y             ||= 100
  args.state.speed         ||= 1
  args.state.angle         ||= 0
end

def rotation_anchor
  [0.7, 0.5]
end

def opacity
  255
end

def saturation
  [255, 255, 255]
end

def destination_rect state
  [state.x, state.y,
   state.sprite.width  * state.sprite.scale,
   state.sprite.height * state.sprite.scale]
end

def source_rect state
  [0, 0, state.sprite.width, state.sprite.height]
end

def render grid, outputs, state
  outputs.solids  <<  [grid.rect, 70, 70, 70]
  outputs.sprites <<  [destination_rect(state),
                       'sprites/86.png',
                       state.angle,
                       opacity,
                       saturation,
                       source_rect(state),
                       false, false,  # don't flip sprites
                       rotation_anchor]

  # also look at the create_sprite helper method
  #
  # For example:
  #
  # dest   = destination_rect(state)
  # source = source_rect(state),
  # outputs.sprites << create_sprite(
  #   'sprites/86.png',
  #   x: dest.x,
  #   y: dest.y,
  #   w: dest.w,
  #   h: dest.h,
  #   angle: state.angle,
  #   source_x: source.x,
  #   source_y: source.y,
  #   source_w: source.w,
  #   source_h: source.h,
  #   flip_h: false,
  #   flip_v: false,
  #   rotation_anchor_x: 0.7,
  #   rotation_anchor_y: 0.5
  # )
end

def calc state
  calc_pos state
  calc_wrap state
end

def calc_pos state
  state.x     += state.angle.vector_x * state.speed
  state.y     += state.angle.vector_y * state.speed
  state.speed *= 1.1
  state.speed  = state.speed.lesser(state.max_speed)
end

def calc_wrap state
  state.x = -state.sprite.width * state.sprite.scale if state.x - 20 > 1280
  state.x = 1280 if state.x + state.sprite.width * state.sprite.scale + 20 < 0
  state.y = 0    if state.y - 20 > 720
  state.y = 720  if state.y + state.sprite.height * state.sprite.scale + 20 < 0
end

def process_inputs args
  if args.inputs.keyboard.key_held.left
    args.state.angle += 2
  elsif args.inputs.keyboard.key_held.right
    args.state.angle -= 2
  elsif args.inputs.controller_one.left_analog_x_perc != 0
    args.state.angle += 2 * args.inputs.controller_one.left_analog_x_perc * -1
  end
end

def tick args
  default args
  render args.grid, args.outputs, args.state
  calc args.state
  process_inputs args
end

def create_sprite path, options = {}
  options = {
    x: 0,
    y: 0,
    w: 100,
    h: 100,

    angle: 0,
    rotation_anchor_x: 0.5,
    rotation_anchor_y: 0.5,

    r: 255,
    g: 255,
    b: 255,
    a: 255,

    source_x: 0,
    source_y: 0,
    source_w: -1,
    source_h: -1,

    flip_h: false,
    flip_v: false,

  }.merge options

  [
    options[:x], options[:y], options[:w], options[:h],
    path,
    options[:angle], options[:a], options[:r], options[:g], options[:b],
    options[:source_x], options[:source_y], options[:source_w], options[:source_h],
    options[:flip_h], options[:flip_v],
    options[:rotation_anchor_x], options[:rotation_anchor_y],
  ]
end
