class Pong
  attr_accessor :grid, :inputs, :state, :outputs, :gtk

  def defaults
    state.ball.debounce   ||= 3 * 60
    state.ball.s          ||= 10
    state.ball.s_half     ||= state.ball.s / 2
    state.ball.x          ||= grid.w_half
    state.ball.y          ||= grid.h_half
    state.ball.dx         ||= 5.randomize(:sign)
    state.ball.dy         ||= 5.randomize(:sign)
    state.player_1_paddle ||= grid.h_half
    state.player_2_paddle ||= grid.h_half
    state.paddle.h        ||= 120
    state.paddle.w        ||= 10
    state.paddle.h_half   ||= 60
    state.player_1_score  ||= 0
    state.player_2_score  ||= 0
  end

  def player_1_paddle_sprite
    [0,
     state.player_1_paddle - state.paddle.h_half,
     state.paddle.w,
     state.paddle.h]
  end

  def player_2_paddle_sprite
    [grid.w - state.paddle.w,
     state.player_2_paddle - state.paddle.h_half,
     state.paddle.w,
     state.paddle.h]
  end

  def ball_sprite
    [state.ball.x - state.ball.s_half,
     state.ball.y - state.ball.s_half,
     state.ball.s,
     state.ball.s]
  end

  def reset_round
    state.ball.x = grid.w_half
    state.ball.y = grid.h_half
    state.ball.dx = 5.randomize(:sign)
    state.ball.dy = 5.randomize(:sign)
    state.ball.debounce = 3 * 60
  end

  def calc
    state.ball.debounce -= 1 and return if state.ball.debounce > 0
    state.ball.x += state.ball.dx
    state.ball.y += state.ball.dy

    if player_2_paddle_sprite.intersects_rect? ball_sprite
      state.ball.dx *= -1
    elsif state.ball.x > grid.w
      state.player_1_score += 1
      reset_round
    end

    if player_1_paddle_sprite.intersects_rect? ball_sprite
      state.ball.dx *= -1
    elsif state.ball.x < 0
      state.player_2_score += 1
      reset_round
    end

    if state.ball.y.add(state.ball.s_half) > grid.h
      state.ball.y = grid.h.subtract(state.ball.s_half)
      state.ball.dy *= -1
    elsif state.ball.y.subtract(state.ball.s_half) < 0
      state.ball.y = state.ball.s_half
      state.ball.dy *= -1
    end
  end

  def render
    outputs.lines  << [grid.w_half, 0, grid.w_half, grid.h]
    outputs.labels << [grid.w_half.fdiv(2),
                       grid.top - 80,
                       state.player_1_score, 10, 1]
    outputs.labels << [grid.w_half.fdiv(2).add(grid.w_half),
                       grid.top - 80,
                       state.player_2_score, 10, 1]
    if state.ball.debounce > 0
      outputs.labels << [grid.w_half, grid.h_half,
                         "%.2f" % state.ball.debounce.fdiv(60).round(2), 10, 1]
    end
    outputs.solids << ball_sprite
    outputs.solids << player_1_paddle_sprite
    outputs.solids << player_2_paddle_sprite
  end

  def input
    if inputs.controller_one.key_down.down || inputs.keyboard.key_down.down
      state.player_1_paddle -= 40
    elsif inputs.controller_one.key_down.up || inputs.keyboard.key_down.up
      state.player_1_paddle += 40
    end

    if inputs.controller_two.key_down.down
      state.player_2_paddle -= 40
    elsif inputs.controller_two.key_down.up
      state.player_2_paddle += 40
    end
  end

  def tick
    defaults
    render
    calc
    input
  end
end

$pong = Pong.new

def tick args
  $pong.grid, $pong.inputs, $pong.state, $pong.outputs, $pong.gtk = args.destructure
  $pong.tick
end
